#!/usr/bin/env ruby

###
## Copyright 2009 by Adron Systems LLC, Laporte, MN  56461.
## Permission is granted to copy, distribute and/or modify this document.
##
##   Purpose: Ion chromatogram RPN calculator.
##            Output is sent to stdout or output file.
##
##  Location: Adron\Engine\Macros.Ruby
##  Filename: ChromCalc.rb
##
##   Started: 2009/06/01
##   Updated:[2016/04/01]
###

# Load the Vx Extract interface library.
require 'ExtractLoad'


###
## Routine displayHelp :
###

def displayHelp
  system("cls")
  printf("Ion chromatogram RPN calculator.  (Extract release : %s)\n", ASE::EXTRACT_RELEASE) ;
  puts
  printf("  %s [InFileName] [OutFileName] /calc=['RPN calculation']\n", File.basename($0))
  print("\t[/sir='scan index range'] [/str='scan time range']\n")
  puts
  print("Options:\n")
  print("  InFileName   File to read (input) is first name specified.\n")
  print("  OutFileName  Output file is second name specified.\n")
  print("               If name not specified, then output goes to stdout.\n")
  puts
  print("  /calc        Ion calculation in RPN notation:\n")
  print("                 Calculate two ion summations, then add together.\n")
  print("                    \"/calc=120..150 189..190 +\" \n")
  print("               Note: Use quotes as shown in the example!\n")
  puts
  print("  /sir         Scan index range:\n")
  print("                  i..j  : Range of scan indices.\n")
  print("               Note: Index values range from zero to the count of scans-1.\n")
  puts
  print("  /str         Scan time range (minutes):\n")
  print("                  r..t  : Range of scan times.\n")
  puts
  print("               Use either /sir or /str, to restrict scan range.\n")
  puts
  print("  Optional items shown in [].\n")
  print("  No parameters generates this help screen.\n")
  puts
  print(ASE::ADRON_COPYRIGHT)
end


###
## Routine displayComments :
###

def displayComments( rdr, fout )
  fn = File.expand_path(rdr.nameOfFile)
  fn.gsub!( File::SEPARATOR, File::ALT_SEPARATOR )
  fout.printf("## Source file  -- %s\n", fn)
  fout.printf("## Vx Extract   -- %s\n", ASE::EXTRACT_RELEASE)

  tm_now = ASE::Time.new
  fout.printf("## Generated on   %s UTC\n", tm_now.image(ASE::Time.FORMAT_VECTOR2_DATETIME))
end


###
## Routine to_minutes :
##
##      Convert passed string to minutes value.
##      Either minutes.fraction or minutes:seconds.fraction format assumed.
###

def to_minutes( str_min )
  value = 0
  case str_min
  when /\:/
    value  = $`.to_f
    value += $'.to_f / 60.0

  when /\.\d*[1-9]/
    value = str_min.to_f

  else
    value = str_min.to_i
  end

  return value
end


###
## Class Parameters :
##
##      Processing variables by parsing input (commandline) arguments.
###

class Parameters
  def initialize( argv )
    @fname_in = @fname_out = nil
    @calc = nil
    @help = @false
    @scan_range = nil
    @time_range = nil

    argv.each do |item|
      case item
      when /^\//
        case item
        when /\/calc(\=|\:)/i
          @calc=$'

        when /\/sir(\=|\:)/i
          values=$'.split(/[ \t\,]+/)
          values.each do |value|
            case value
            when /\.\./
              @scan_range = Range.new($`.to_i, $'.to_i)
            else
              raise 'Syntax error on /sir parameter.'
            end
          end

        when /\/str(\=|\:)/i
          values=$'.split(/[ \t\,]+/)
          values.each do |value|
            case value
            when /\.\./
              @time_range = Range.new(to_minutes($`), to_minutes($'))
            else
              raise 'Syntax error on /str parameter.'
            end
          end

        when /\/help/i
          @help = true
        end
      else
        if (@fname_in.nil?)
          @fname_in = item
        else
          @fname_out = item
        end
      end
    end
  end

  attr_reader :fname_in, :fname_out
  attr_reader :calc, :help
  attr_reader :scan_range, :time_range
end


###
## Entrance :
###

# Parse the passed arguments.
parms = Parameters.new( ARGV )
##print(parms.inspect, "\n")

# No arguments or help flag, then display help screen.
# No further processing if help screen displayed.
if (ARGV.empty?) || (parms.help) || (parms.calc.nil?)
  displayHelp
  Process.exit
end

# Open a ASE::Reader object.
begin
  rdr = ASE::Reader.new( parms.fname_in )
rescue Exception
  raise "Unable to create ASE::Reader object."
end

# Output to file or $stdout
fout = (!parms.fname_out.nil?) ? File.open(parms.fname_out, "w+") : $stdout

# Display various comments.
displayComments( rdr, fout )
fout.puts

# Determine range of scan indices to iterate over.
if ( !parms.scan_range.nil? )
  rg_scans = parms.scan_range
elsif ( !parms.time_range.nil? )
  idx_left  = rdr.scanAtInterval( false, parms.time_range.first * 60.0 )
  idx_right = rdr.scanAtInterval( false, parms.time_range.last  * 60.0 )
  rg_scans  = (idx_left..idx_right)
else
  rg_scans=0...rdr.numberOfScans
end

# Loop through scans indices and do the RPN ion calculation.
stack = Array.new
items = parms.calc.split(/[ \t]+/)
rg_scans.each do |i|
  stack.clear

  items.each do |item|
    case item
      when /\d+/        # Single ion as an integer.
        stack.push( rdr.extractIon(i, item.to_i) )

      when /\.\./       # Summation as a range of ions.
        stack.push( rdr.extractIon(i, Range.new(($`.to_i), ($'.to_i)) ) )

      when /tic/i       # Special value, the TIC summation.
        stack.push( rdr.summationOfScan(i) )

      when /\S*/        # Remainder assumed to be operator.
        raise 'RPN syntax error.' if stack.size < 2

        second_operand = stack.pop()
        first_operand = stack.pop()

        stack.push(first_operand.send(item, second_operand))
    end

  end

  raise 'RPN syntax error.' if stack.size != 1

  # Interval of scan (seconds) plus ion calculation value.
  fout.printf("%.6f\t%g\n", rdr.intervalOfScan(i), stack.pop() )
end


###
## For further information on the RPN calculation, please see the
## following:
##
##      http://lukaszwrobel.pl/blog/reverse-polish-notation-parser
###
