#!/usr/bin/env ruby

###
## Copyright 2009, 2012 by Adron Systems LLC, Laporte, MN  56461.
## Permission is granted to copy, distribute and/or modify this document.
##
##   Purpose: Extract one or more spectra from the input data file.
##            Output is sent to stdout or output file.
##
##  Location: Adron\Engine\Macros.Ruby
##  Filename: Spectra.rb
##
##   Started: 2009/06/01
##   Updated:[2016/04/02]
###

# Load the Vx Extract interface library.
require 'ExtractLoad'


###
## Routine displayHelp :
###

def displayHelp
  system("cls")
  printf("Extract one or more spectra from data file.  (Extract release : %s)\n", ASE::EXTRACT_RELEASE) ;
  puts
  printf("  %s [InFileName] [OutFileName] \n", File.basename($0))
  print("\t[/sil='scan index list'] [/stl='scan time list'] [/nominal] [/percent]\n")
  puts
  print("Options:\n")
  print("  InFileName   File to read (input) is first name specified.\n")
  print("  OutFileName  Output file is second name specified.\n")
  print("               If name not specified, then output goes to stdout.\n")
  puts
  print("  /sil         Scan index list in various forms:\n")
  print("                  i     : A single scan index.\n")
  print("                  i..j  : A range of scan indices.\n")
  print("                  i,j,k : A list of scan indices, comma separated.\n")
  print("               Note: Index values range from zero to the count of scans-1.\n")
  puts
  print("  /stl         Scan time list (minutes) in various forms:\n")
  print("                  r     : A single scan time.\n")
  print("                  r..t  : A range of scan times.\n")
  print("                  r,s,t : A list of scan times, comma separated.\n")
  puts
  print("  /nominal     Specifies whether nominal masses should be used.\n")
  print("  /percent     Specifies whether to generate percent of base peak column.\n")
  puts
  print("  Optional items shown in [].\n")
  print("  No parameters generates this help screen.\n")
  puts
  print(ASE::ADRON_COPYRIGHT)
end


###
## Routine displayComments :
###

def displayComments( rdr, fout )
  fn = File.expand_path(rdr.nameOfFile)
  fn.gsub!( File::SEPARATOR, File::ALT_SEPARATOR )
  fout.printf("## Source file  -- %s\n", fn)
  fout.printf("## Vx Extract   -- %s\n", ASE::EXTRACT_RELEASE)

  tm_now = ASE::Time.new
  fout.printf("## Generated on   %s UTC\n", tm_now.image(ASE::Time.FORMAT_VECTOR2_DATETIME))
end


###
## Routine displaySpectrum :
##
##      Each spectrum is introduced with its scan interval value.
##      A listing of mass intensity pairs follows.
###

def displaySpectrum( rdr, nominal, percent, i, spectrum, fout )
  # Extract scan information.
  pk_base   = rdr.basePeakOfScan(nominal, i)
  itv_scan  = rdr.intervalOfScan( i )
  cnt_peaks = rdr.extractScan(nominal, i, spectrum)

  # Output scan index plus interval of scan in seconds.
  fout.printf("IndexOfScan=%d\n", i )
  fout.printf("IntervalOfScan=%.6f\n", itv_scan )

  # Generate format string for mass intensity pairs.
  str_frmt = "%d,%d"
  if (!nominal)
    mass_prec = (Math.log10(ASE::MI::MASS_SCALAR)).to_i
    str_frmt = sprintf("%%.%df,%%d", mass_prec)
  end

  # Output mass / intensity pairs as CSV.
  rg_pairs = (0...cnt_peaks)
  rg_pairs.each do |j|
    fout.printf(str_frmt, spectrum[j].mass, spectrum[j].intensity)
    if (percent && (0 < pk_base.intensity))
      fout.printf(",%.1f", 100.0 * spectrum[j].intensity / pk_base.intensity)
    end
    fout.puts
  end
end


###
## Routine to_minutes :
##
##      Convert passed string to minutes value.
##      Either minutes.fraction or minutes:seconds.fraction format assumed.
###

def to_minutes( str_min )
  value = 0
  case str_min
  when /\:/
    value  = $`.to_f
    value += $'.to_f / 60.0

  when /\.\d*[1-9]/
    value = str_min.to_f

  else
    value = str_min.to_i
  end

  return value
end


###
## Class Parameters :
##
##      Processing variables by parsing input (commandline) arguments.
###

class Parameters
  def initialize( argv )
    @fname_in = @fname_out = nil
    @help = @nominal = @percent = false
    @scan_indices = Array.new
    @time_indices = Array.new

    argv.each do |item|
      case item
      when /^\//
        case item
        when /\/sil(\=|\:)/i
          values=$'.split(/[ \t\,]+/)
          values.each do |value|
            case value
            when /\.\./
              @scan_indices << Range.new($`.to_i, $'.to_i)
            else
              @scan_indices << value.to_i
            end
          end

        when /\/stl(\=|\:)/i
          values=$'.split(/[ \t\,]+/)
          values.each do |value|
            case value
            when /\.\./
              @time_indices << Range.new(to_minutes($`), to_minutes($'))

            else
              @time_indices << to_minutes(value)
            end
          end

        when /\/nom/i
          @nominal = true

        when /\/percent/i
          @percent = true

        when /\/help/i
          @help = true
        end
      else
        if (@fname_in.nil?)
          @fname_in = item
        else
          @fname_out = item
        end
      end
    end
  end

  attr_reader :fname_in, :fname_out
  attr_reader :nominal, :percent, :help
  attr_reader :scan_indices, :time_indices
end


###
## Entrance :
###

# Parse the passed arguments.
parms = Parameters.new( ARGV )
##print(parms.inspect, "\n")

# No arguments or help flag, then display help screen.
# No further processing if help screen displayed.
if (ARGV.empty?) || (parms.help)
  displayHelp
  Process.exit
end

# Open a ASE::Reader object.
begin
  rdr = ASE::Reader.new( parms.fname_in )
rescue Exception
  raise "Unable to create ASE::Reader object."
end

# Output to file or $stdout
fout = (!parms.fname_out.nil?) ? File.open(parms.fname_out, "w+") : $stdout

# Display various comments.
displayComments( rdr, fout )
fout.puts

# Spectrum storage buffer.
spectrum=Array.new(50)

# Indices for scan range.
rg_scans=Range.new(0,0)

# Loop through scans by scan indices.
parms.scan_indices.each do |item|
  rg_scans = (item.class == Range) ? item : item..item
  rg_scans.each do |i|
    # Display the spectrum.
    displaySpectrum( rdr, parms.nominal, parms.percent, i, spectrum, fout )
    fout.puts
  end
end

# Loop through scans by time indices.
parms.time_indices.each do |item|
  if (item.class == Range)
    rg_scans = rdr.scanAtInterval( false, item.first * 60.0 )..rdr.scanAtInterval( true, item.last * 60.0 )
  else
    iscan = rdr.scanAtInterval( item * 60.0 )
    rg_scans = iscan..iscan
  end
  rg_scans.each do |i|
    # Display the spectrum.
    displaySpectrum( rdr, parms.nominal, parms.percent, i, spectrum, fout )
    fout.puts
  end
end
